@@ -1,73 +1,28 @@
-//! Contents
-//! ========
-//! - [Introduction](#introduction)
-//! - [Features](#features)
-//! - [Usage](#usage)
-//! ## Introduction
-//! GemView is a [gemini protocol](https://gemini.circumlunar.space/) browser widget
-//! for gtk+ (version 4) implemented in Rust.
-//! ## Features
-//! - [x] Browse and render gemini gemtext content
-//! - [x] Display plain text over gemini
-//! - [x] Display images over gemini
-//! - [x] Display text and images from `data://` url's
-//! - [x] Browse and render gopher maps, plain text and images over gopher
-//! - [x] Display finger protocol content
-//! - [x] Browse local files and directories via 'file://' url's
-//! - [x] Open http(s) links in a *normal* browser
-//! - [x] User customizable fonts
-//! - [x] User customizable colors (via CSS)
-//! - [x] Back/forward list
-//! - [ ] History
-//!
-//! ## Usage
-//! ```Yaml
-//! [dependencies]
-//! gemview = 0.2.0
-//!
-//! [dependencies.gtk]
-//! version = "~0.4"
-//! package = "gtk4"
-//! ```
-//! ```Rust
-//! use gemview::GemView;
-//! use gtk::prelude::*;
-//!
-//! let browser = GemView::default();
-//! let scroller = gtk::builders::ScrolledWindowBuilder::new()
-//!     .child(&browser)
-//!     .hexpand(true)
-//!     .vexpand(true)
-//!     .build();
-//! let window = gtk::builders::WindowBuilder::new()
-//!     .child(&scroller)
-//!     .title("GemView")
-//!     .build()
-//! window.show();
-//! browser.visit("gemini://gemini.circumlunar.space");
-//! ```
+#![doc = include_str!("../README.md")]
 
-use glib::Object;
-use glib::{Continue, MainContext, PRIORITY_DEFAULT};
-use gtk::gdk_pixbuf::Pixbuf;
-use gtk::gio::{Cancellable, MemoryInputStream, Menu, MenuItem, SimpleAction, SimpleActionGroup};
-use gtk::glib;
-use gtk::pango::FontDescription;
-use gtk::prelude::*;
-use gtk::subclass::prelude::*;
-use textwrap::fill;
-use url::Url;
+use {
+    glib::{Continue, MainContext, Object, PRIORITY_DEFAULT},
+    gtk::{
+        glib,
+        gdk_pixbuf::Pixbuf,
+        gio::{Cancellable, MemoryInputStream, Menu, MenuItem, SimpleAction, SimpleActionGroup},
+        pango::FontDescription,
+        prelude::*,
+        subclass::prelude::*,
+    },
+    textwrap::fill,
+    url::Url,
+    std::{path::PathBuf, thread},
+};
 
-use std::path::PathBuf;
-use std::thread;
-
-mod scheme;
-use gemini::parser::GemtextNode;
-use scheme::data::{Data, DataUrl, MimeType};
-use scheme::gopher::GopherMap;
-use scheme::{finger, gemini, gopher};
-use scheme::{Content, Response};
 mod imp;
+pub mod scheme;
+use {
+    scheme::{Content, data, finger, gemini, gopher, Response},
+    data::{Data, DataUrl, MimeType},
+    gemini::parser::GemtextNode,
+    gopher::GopherMap,
+};
 
 glib::wrapper! {
 /// The gemini browser widget is a subclass of the `TextView` widget which
@@ -105,7 +60,7 @@ impl GemView {
         request_new_tab.connect_activate(move |_, url| {
             if let Some(url) = url {
                 if let Some(url) = url.get::<String>() {
-                    if let Ok(url) = urlencoding::decode(&url.to_string()) {
+                    if let Ok(url) = urlencoding::decode(&url) {
                         if let Ok(url) = viewer.absolute_url(&url) {
                             viewer.emit_by_name::<()>("request-new-tab", &[&url.to_string()]);
                         }
@@ -117,7 +72,7 @@ impl GemView {
         request_new_window.connect_activate(move |_, url| {
             if let Some(url) = url {
                 if let Some(url) = url.get::<String>() {
-                    if let Ok(url) = urlencoding::decode(&url.to_string()) {
+                    if let Ok(url) = urlencoding::decode(&url) {
                         if let Ok(url) = viewer.absolute_url(&url) {
                             viewer.emit_by_name::<()>("request-new-window", &[&url.to_string()]);
                         }
@@ -178,7 +133,7 @@ impl GemView {
     /// will not need to be called directly.
     pub fn append_history(&self, uri: &str) {
         let current = self.uri();
-        if &current != uri {
+        if current != uri {
             self.imp().history.borrow_mut().append(uri.to_string());
         }
     }
@@ -536,7 +491,7 @@ impl GemView {
                         .build();
                     label.set_cursor_from_name(Some("pointer"));
                     let open_menu = Menu::new();
-                    let ln = format!("gopher://{}:{}{}", &link.host, &link.port, &link.path,);
+                    let ln = format!("gopher://{}:{}{}", &link.host, &link.port, &link.path);
                     let ln = urlencoding::encode(&ln);
                     let action_name = format!("viewer.request-new-tab('{}')", &ln,);
                     let in_tab = MenuItem::new(Some("Open in new tab"), Some(&action_name));
@@ -554,6 +509,29 @@ impl GemView {
                         gtk::Inhibit(true)
                     });
                 }
+                gopher::parser::LineType::Query(link) => {
+                    let anchor = buf.create_child_anchor(&mut iter);
+                    let label = gtk::builders::LabelBuilder::new()
+                        .use_markup(true)
+                        .tooltip_text(&format!(
+                            "gopher://{}:{}{}",
+                            &link.host, &link.port, &link.path,
+                        ))
+                        .label(&link.to_markup(&self.font_pre()))
+                        .build();
+                    label.set_cursor_from_name(Some("pointer"));
+                    self.add_child_at_anchor(&label, &anchor);
+                    iter = buf.end_iter();
+                    buf.insert(&mut iter, "\n");
+                    let viewer = self.clone();
+                    label.connect_activate_link(move |_, link| {
+                        viewer.emit_by_name::<()>(
+                            "request-input",
+                            &[&String::from("Enter query"), &link],
+                        );
+                        gtk::Inhibit(true)
+                    });
+                }
             }
         }
     }
@@ -577,7 +555,7 @@ impl GemView {
             Err(e) => match e {
                 url::ParseError::RelativeUrlWithoutBase => {
                     let origin = url::Url::parse(&self.uri())?;
-                    let new = origin.join(&url)?;
+                    let new = origin.join(url)?;
                     Ok(new)
                 }
                 _ => Err(e.into()),
@@ -626,7 +604,7 @@ impl GemView {
                     let url = url.to_string();
                     self.append_history(&url);
                     self.set_buffer_mime("text/plain");
-                    self.set_buffer_content(&payload.as_bytes());
+                    self.set_buffer_content(payload.as_bytes());
                     self.emit_by_name::<()>("page-loaded", &[&url]);
                 }
                 _ => unreachable!(),
@@ -637,7 +615,7 @@ impl GemView {
                     let url = url.to_string();
                     self.append_history(&url);
                     self.set_buffer_mime("text/gemini");
-                    self.set_buffer_content(&payload.as_bytes());
+                    self.set_buffer_content(payload.as_bytes());
                     self.emit_by_name::<()>("page-loaded", &[&url]);
                 }
                 _ => unreachable!(),
@@ -715,7 +693,6 @@ impl GemView {
     fn load_gopher(&self, url: Url) {
         let (sender, receiver) = MainContext::channel(PRIORITY_DEFAULT);
         let req = url.clone();
-        let sender = sender.clone();
         thread::spawn(move || match gopher::request(&req) {
             Ok(content) => {
                 sender
@@ -748,6 +725,14 @@ impl GemView {
                         let url = url.to_string();
                         viewer.append_history(&url);
                         viewer.emit_by_name::<()>("page-loaded", &[&url]);
+                    } else {
+                        let filename = if let Some(segments) = url.path_segments() {
+                            segments.last().unwrap_or("download")
+                        } else {
+                            "download"
+                        }.to_string();
+
+                        viewer.emit_by_name::<()>("request-download", &[&content.mime, &filename]);
                     }
                 }
                 Response::Error(err) => {
@@ -761,7 +746,6 @@ impl GemView {
     fn load_finger(&self, url: Url) {
         let (sender, receiver) = MainContext::channel(PRIORITY_DEFAULT);
         let req = url.clone();
-        let sender = sender.clone();
         thread::spawn(move || match finger::request(&req) {
             Ok(content) => {
                 sender
@@ -795,9 +779,9 @@ impl GemView {
 
     fn load_gemini(&self, url: Url) {
         let (sender, receiver) = MainContext::channel(PRIORITY_DEFAULT);
-        let sender = sender.clone();
+        let u = url.clone();
         thread::spawn(move || {
-            let mut url = url;
+            let mut url = u;
             loop {
                 let response = match gemini::request::make_request(&url) {
                     Ok(r) => r,
@@ -895,7 +879,12 @@ impl GemView {
                             viewer.emit_by_name::<()>("page-loaded", &[&content.url]);
                         }
                         _ => {
-                            viewer.emit_by_name::<()>("request-download", &[&content.mime]);
+                            let filename = if let Some(segments) = url.path_segments() {
+                                segments.last().unwrap_or("download")
+                            } else {
+                                "download"
+                            }.to_string();
+                            viewer.emit_by_name::<()>("request-download", &[&content.mime, &filename]);
                         }
                     }
                 }
@@ -982,6 +971,21 @@ impl GemView {
         })
     }
 
+    /// Connects to the "request-download" signal, emitter when the browser has
+    /// encountered a request for a file type it does not know how to render
+    pub fn connect_request_download<F: Fn(&Self, String, String) + 'static>(
+        &self,
+        f: F,
+    ) -> glib::SignalHandlerId {
+        self.connect_local("request-download", true, move |values| -> Option<glib::Value> {
+            let obj = values[0].get::<Self>().unwrap();
+            let mime = values[1].get::<String>().unwrap();
+            let filename = values[2].get::<String>().unwrap();
+            f(&obj, mime, filename);
+            None
+        })
+    }
+
     /// Connects to the "request-new-tab" signal, emitted when the "Open in new
     /// tab" item is chosen from the context menu for link items.
     pub fn connect_request_new_tab<F: Fn(&Self, String) + 'static>(
