@@ -9,8 +9,16 @@ pub struct Content {
     pub bytes: Vec<u8>,
 }
 
+#[derive(Clone, Debug)]
+pub struct Input {
+    pub meta: String,
+    pub url: String,
+    pub sensitive: u8,
+}
+
 #[derive(Clone, Debug)]
 pub enum Response {
     Success(Content),
+    RequestInput(Input),
     Error(String),
 }
